package org.xanot;

import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import org.xanot.annotation.TagInstance;
import org.xanot.annotation.XmlAttribute;
import org.xanot.annotation.XmlCollectionProperty;
import org.xanot.annotation.XmlSingleProperty;
import org.xanot.dtd.DtdElement;
import org.xanot.dtd.DtdModel;

public class XanotModelToXml {
	private boolean withCdata = true;

	public boolean isWithCdata() {
		return withCdata;
	}

	public void setWithCdata(boolean withCdata) {
		this.withCdata = withCdata;
	}

	public static void writeXmlFromModel(OutputStream os, Object model,
			DtdModel dtd, boolean withCData) throws IOException, XanotException {
		XanotModelToXml mtx = new XanotModelToXml();
		mtx.setWithCdata(withCData);
		mtx.writeXmlFromModel(os, model, dtd);
	}

	public void writeXmlFromModel(OutputStream os, Object model, DtdModel dtd)
			throws IOException, XanotException {
		Tag modelTag = toXmlGenericModel(model, dtd);
		modelTag.write(os);
	}

	private Tag toXmlGenericModel(Object o, DtdModel dtd) throws XanotException {
		if (o.getClass().isAnnotationPresent(TagInstance.class)) {
			TagInstance ti = (TagInstance) o.getClass().getAnnotation(
					TagInstance.class);
			Tag tag = new Tag(ti.path());
			tag.getAttributes().putAll(getAttributes(o));
			setNativeAttributes(tag, o);
			setNonNativeAttributes(tag, o, dtd);
			setNativeCollectionAttributes(tag, o);
			setNonNativeCollectionAttributes(tag, o, dtd);
			return tag;
		} else {
			throw new XanotException(
					"Class "
							+ o.getClass().getName()
							+ " is reference in model but its not annotated by @TagInstance");
		}
	}

	private void setNonNativeCollectionAttributes(Tag tag, Object o,
			DtdModel dtd) throws XanotException {
		for (Method method : o.getClass().getMethods()) {
			try {
				if ((method.getName().startsWith("is") || method.getName()
						.startsWith("get"))
						&& method
								.isAnnotationPresent(XmlCollectionProperty.class)) {
					XmlCollectionProperty xcp = (XmlCollectionProperty) method
							.getAnnotation(XmlCollectionProperty.class);
					if (!AnnotationRuleBuilder.isNativeWrapper(xcp.element())) {
						if (Set.class.isAssignableFrom(method.getReturnType())) {
							for (Object obj : (Set) method.invoke(o,
									new Object[0])) {
								tag.getChild().add(toXmlGenericModel(obj, dtd));
							}
						} else if (List.class.isAssignableFrom(method
								.getReturnType())) {
							for (Object obj : (List) method.invoke(o,
									new Object[0])) {
								tag.getChild().add(toXmlGenericModel(obj, dtd));
							}
						}
					}
				}
			} catch (IllegalArgumentException e) {
				throw new XanotException(e);
			} catch (IllegalAccessException e) {
				throw new XanotException(e);
			} catch (InvocationTargetException e) {
				throw new XanotException(e);
			}
		}
	}

	private void setNativeCollectionAttributes(Tag tag, Object o)
			throws XanotException {
		for (Method method : o.getClass().getMethods()) {
			try {
				if ((method.getName().startsWith("is") || method.getName()
						.startsWith("get"))
						&& method
								.isAnnotationPresent(XmlCollectionProperty.class)) {
					XmlCollectionProperty xcp = (XmlCollectionProperty) method
							.getAnnotation(XmlCollectionProperty.class);
					if (AnnotationRuleBuilder.isNativeWrapper(xcp.element())) {
						String key = xcp.xmltag();
						if (Set.class.isAssignableFrom(method.getReturnType())) {
							for (Object obj : (Set) method.invoke(o,
									new Object[0])) {
								Tag tagTwo = new Tag(key);
								tagTwo.setCharacters(obj.toString());
								tag.getChild().add(tagTwo);
							}
						} else if (List.class.isAssignableFrom(method
								.getReturnType())) {
							for (Object obj : (List) method.invoke(o,
									new Object[0])) {
								Tag tagTwo = new Tag(key);
								tagTwo.setCharacters(obj.toString());
								tag.getChild().add(tagTwo);
							}
						}
					}
				}
			} catch (IllegalArgumentException e) {
				throw new XanotException(e);
			} catch (IllegalAccessException e) {
				throw new XanotException(e);
			} catch (InvocationTargetException e) {
				throw new XanotException(e);
			}
		}
	}

	private void setNonNativeAttributes(Tag tag, Object o, DtdModel dtd)
			throws XanotException {
		for (Method method : o.getClass().getMethods()) {
			if ((method.getName().startsWith("is") || method.getName()
					.startsWith("get"))
					&& method.isAnnotationPresent(XmlSingleProperty.class)
					&& !AnnotationRuleBuilder.isNativeWrapper(method
							.getReturnType())) {
				if (method.getReturnType().isArray()) {
					throw new XanotException(
							"Cannot process method with array return. "
									+ o.getClass().getName() + "."
									+ method.getName());
				}
				tag.getChild().add(toXmlGenericModel(o, dtd));
			}
		}
	}

	private void setNativeAttributes(Tag tag, Object o) throws XanotException {
		for (Method method : o.getClass().getMethods()) {
			try {
				if ((method.getName().startsWith("is") || method.getName()
						.startsWith("get"))
						&& method.isAnnotationPresent(XmlSingleProperty.class)
						&& AnnotationRuleBuilder.isNativeWrapper(method
								.getReturnType())) {
					if (method.getReturnType().isArray()) {
						throw new XanotException(
								"Cannot process method with array return. "
										+ o.getClass().getName() + "."
										+ method.getName());
					}
					XmlSingleProperty xsp = (XmlSingleProperty) method
							.getAnnotation(XmlSingleProperty.class);
					String key = xsp.name();
					Object value = method.invoke(o, new Object[0]);
					Tag tagTwo = new Tag(key);
					tagTwo.setCharacters(value.toString());
					tag.getChild().add(tagTwo);
				}
			} catch (IllegalArgumentException e) {
				throw new XanotException(e);
			} catch (IllegalAccessException e) {
				throw new XanotException(e);
			} catch (InvocationTargetException e) {
				throw new XanotException(e);
			}
		}
	}

	private HashMap<String, Object> getAttributes(Object o)
			throws XanotException {
		HashMap<String, Object> attr = new HashMap<String, Object>();
		for (Method method : o.getClass().getMethods()) {
			try {
				if ((method.getName().startsWith("is") || method.getName()
						.startsWith("get"))
						&& method.isAnnotationPresent(XmlAttribute.class)) {
					XmlAttribute xa = (XmlAttribute) method
							.getAnnotation(XmlAttribute.class);
					String key = xa.name();
					Object value = method.invoke(o, new Object[0]);
					attr.put(key, value);
				}
			} catch (IllegalArgumentException e) {
				throw new XanotException(e);
			} catch (IllegalAccessException e) {
				throw new XanotException(e);
			} catch (InvocationTargetException e) {
				throw new XanotException(e);
			}
		}
		return attr;
	}

	private class TagDtdComparator implements Comparator {

		private DtdElement element;

		private String[] sequence;

		public TagDtdComparator(DtdModel dtdModel, String baseTag) {
			super();
			element = dtdModel.getDtdElement(baseTag);
			if (element == null) {
				throw new IllegalArgumentException("No element " + baseTag
						+ " in dtd");
			}
			sequence = element.extractWords(element.getCategory());
			// FIXME Finish this comparator later. Ive got not time. Got to go
			// home.
		}

		public int compare(Object o1, Object o2) {
			return 0;
		}

	}

	private class Tag implements Comparable<Tag> {
		private String name = null;

		private HashMap<String, Object> attributes = new HashMap<String, Object>();

		private ArrayList<Tag> child = new ArrayList<Tag>();

		private String characters = null;

		private boolean empty = false;

		private int childIndex = 0;

		public int compareTo(Tag o) {
			return childIndex - o.getChildIndex();
		}

		public int getChildIndex() {
			return childIndex;
		}

		public void setChildIndex(int childIndex) {
			this.childIndex = childIndex;
		}

		public String getCharacters() {
			return characters;
		}

		public void setCharacters(String characters) {
			this.characters = characters;
		}

		public Tag(String name) {
			this.name = name;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public HashMap<String, Object> getAttributes() {
			return attributes;
		}

		public ArrayList<Tag> getChild() {
			return child;
		}

		public void setChild(ArrayList<Tag> child) {
			this.child = child;
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();
			sb.append("<");
			sb.append(getName());
			if (getAttributes().size() > 0) {
				for (String key : getAttributes().keySet()) {
					sb.append(" ");
					sb.append(key);
					sb.append("=");
					sb.append("\"");
					sb.append(getAttributes().get(key));
					sb.append("\"");
				}
			}
			if (isEmpty()) {
				sb.append("/>");
			} else {
				sb.append(">");
				if (getCharacters() != null) {
					if (isWithCdata()) {
						sb.append("<![CDATA[");
						sb.append(getCharacters());
						sb.append("]]>");
					} else {
						sb.append(getCharacters());
					}
				} else if (getChild().size() > 0) {
					Collections.sort(getChild());
					for (Tag sub : getChild()) {
						sb.append(sub.toString());
					}
				}
				sb.append("</");
				sb.append(getName());
				sb.append(">");
			}
			return super.toString();
		}

		public void write(OutputStream os) throws IOException {
			write("<", os);
			write(getName(), os);
			if (getAttributes().size() > 0) {
				for (String key : getAttributes().keySet()) {
					write(" ", os);
					write(key, os);
					write("=", os);
					write("\"", os);
					write(getAttributes().get(key).toString(), os);
					write("\"", os);
				}
			}
			if (isEmpty()) {
				write("/>", os);
			} else {
				write(">", os);
				if (getCharacters() != null) {
					if (isWithCdata()) {
						write("<![CDATA[", os);
						write(getCharacters(), os);
						write("]]>", os);
					} else {
						write(getCharacters(), os);
					}
				} else if (getChild().size() > 0) {
					Collections.sort(getChild());
					for (Tag sub : getChild()) {
						sub.write(os);
					}
				}
				write("</", os);
				write(getName(), os);
				write(">", os);
			}
		}

		public boolean isEmpty() {
			return empty;
		}

		public void setEmpty(boolean empty) {
			this.empty = empty;
		}

		private void write(String s, OutputStream os) throws IOException {
			os.write(s.getBytes());
		}
	}
}
