package org.xanot.dtd;

import java.util.ArrayList;

/**
 * This is the interface model of DTD file structures. DTD file may contains
 * list of elements and attributes. Using this interface model one could browse
 * through the elements and attributes.
 * 
 * @author Ferdinand Neman (newm4n_at_gmail.com)
 * 
 */
public interface DtdModel {

	/**
	 * Get list of attList element of DTD. the Att list is always in sequence as
	 * the dtd file have put it.
	 * 
	 * @return List of DtdAttList model objects.
	 */
	public abstract ArrayList<DtdAttList> getDtdAttLists();

	/**
	 * Get list of elements element of DTD. The Element is always in sequence as
	 * the dtd file have put it.
	 * 
	 * @return List of DtdElement model objects.
	 */
	public abstract ArrayList<DtdElement> getDtdElements();

	/**
	 * Get numbers of attList element within the dtd file.
	 * 
	 * @return Number of attList element inside dtd.
	 */
	public abstract int dtdAttListCount();

	/**
	 * Get numbers of Element within the dtd file.
	 * 
	 * @return Number of Element inside dtd.
	 */
	public abstract int dtdElementCount();

	/**
	 * Get attList model object at specific index.
	 * 
	 * @param idx
	 *            Index
	 * @return DtdAttList object at specified index.
	 */
	public abstract DtdAttList getDtdAttListAt(int idx);

	/**
	 * Get Element model object at specific index.
	 * 
	 * @param idx
	 *            Index
	 * @return DtdAttList object at specific index.
	 */
	public abstract DtdElement getDtdElementAt(int idx);

	/**
	 * Get Element model object by giving its name
	 * 
	 * @param name
	 *            Element name
	 * @return Element.
	 */
	public abstract DtdElement getDtdElement(String name);
}