/*
 * Copyright 2006 The Xanot team
 */
package org.xanot.structure;

/**
 * Attribute setter rule. This rule define how an XML element's attribute tobe
 * mapped to an object's property.
 * 
 * @author Ferdinand Neman (newm4n _at_ gmail.com)
 */
public class AttributeSetRule extends XanotRule {
	private String propertyName = null;

	private String xmlAttributeName = null;

	private String dateFormat = null;

	/**
	 * Creates a new AttributeSetRule object. TODO Consider the usage of the
	 * xmlAttributeName. Should we remove it ?
	 * 
	 * @param path
	 *            The path, should be equals to the parent path. Hmm. what for.
	 * @param propertyName
	 *            The object's property name.
	 * @param xmlAttributeName
	 *            The XML attribute name.
	 */
	public AttributeSetRule(String path, String propertyName,
			String xmlAttributeName) {
		super(path);
		setPropertyName(propertyName);
		setXmlAttributeName(xmlAttributeName);
	}

	/**
	 * Get Object's property name
	 * 
	 * @return Returns the propertyName.
	 */
	public String getPropertyName() {
		return propertyName;
	}

	/**
	 * set Object's property name
	 * 
	 * @param propertyName
	 *            The propertyName to set.
	 */
	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
	}

	/**
	 * @return Returns the xmlAttributeName.
	 */
	public String getXmlAttributeName() {
		return xmlAttributeName;
	}

	/**
	 * @param xmlAttributeName
	 *            The xmlAttributeName to set.
	 */
	public void setXmlAttributeName(String xmlAttributeName) {
		this.xmlAttributeName = xmlAttributeName;
	}

	/**
	 * Get string representation of this object.
	 * 
	 * @return String representation.
	 */
	public String toString() {
		return "AttributeSetRule : [path:" + getPath() + "],[property:"
				+ propertyName + "],[xmlAttributeName:" + xmlAttributeName
				+ "]";
	}

	/**
	 * Get the date format value.
	 * 
	 * @return Date format pattern
	 */
	public String getDateFormat() {
		return dateFormat;
	}

	/**
	 * Set the date format value
	 * 
	 * @param dateFormat
	 *            new date format pattern
	 */
	public void setDateFormat(String dateFormat) {
		this.dateFormat = dateFormat;
	}

}
