/*
 * Copyright 2006 The Xanot team
 */
package org.xanot.structure;

/**
 * Collection adding rule. This rule define how an XML element's attribute tobe
 * mapped to an object's collection property.
 * 
 * @author Ferdinand Neman (newm4n _at_ gmail.com)
 */
public class CloseCollectionInstanceRule extends XanotRule {
	private String methodName = "";

	private String dateFormat = null;

	/**
	 * Creates a new CloseColletionInstance object.
	 * 
	 * @param path
	 *            Path of the element. Equals to xml element name.
	 * @param methodName
	 *            Method name tobe called on the parent object for addition.
	 */
	public CloseCollectionInstanceRule(String path, String methodName) {
		super(path);
		this.methodName = methodName;
	}

	/**
	 * Get the parent's method name used for addition.
	 * 
	 * @return Returns the methodName.
	 */
	public String getMethodName() {
		return methodName;
	}

	/**
	 * Set the parent's method name used for addition.
	 * 
	 * @param methodName
	 *            The methodName to set.
	 */
	public void setMethodName(String methodName) {
		this.methodName = methodName;
	}

	/**
	 * Get string representation of this object.
	 * 
	 * @return String representation.
	 */
	public String toString() {
		return "CloseCollectionInstanceRule : [path:" + getPath()
				+ "],[methodName:" + methodName + "]";
	}

	/**
	 * Get the date format value.
	 * 
	 * @return Date format pattern
	 */
	public String getDateFormat() {
		return dateFormat;
	}

	/**
	 * Set the date format value
	 * 
	 * @param dateFormat
	 *            new date format pattern
	 */
	public void setDateFormat(String dateFormat) {
		this.dateFormat = dateFormat;
	}
}
