/*
 * Copyright 2006 The Xanot team
 */
package org.xanot.structure;

/**
 * Close instance rule. This rule define how an XML element's element tobe set
 * into the parent property.
 * 
 * @author Ferdinand Neman (newm4n _at_ gmail.com)
 */
public class CloseInstanceRule extends XanotRule {
	private String propertyName = null;

	private String dateFormat = null;

	/**
	 * Creates a new CloseInstanceRule object.
	 * 
	 * @param path
	 *            The path, should be equals to the closing xml tag name.
	 * @param propertyName
	 *            Property on the parent side where the object belongs.
	 */
	public CloseInstanceRule(String path, String propertyName) {
		super(path);
		setPropertyName(propertyName);
	}

	/**
	 * Get property on the parent side where the object belongs.
	 * 
	 * @return Returns the propertyName.
	 */
	public String getPropertyName() {
		return propertyName;
	}

	/**
	 * Set property on the parent side where the object belongs.
	 * 
	 * @param propertyName
	 *            The propertyName to set.
	 */
	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
	}

	/**
	 * Get string representation of this object.
	 * 
	 * @return String representation.
	 */
	public String toString() {
		return "CloseInstanceRule : [path:" + getPath() + "],[property:"
				+ propertyName + "]";
	}

	/**
	 * Get the date format value.
	 * 
	 * @return Date format pattern
	 */
	public String getDateFormat() {
		return dateFormat;
	}

	/**
	 * Set the date format value
	 * 
	 * @param dateFormat
	 *            new date format pattern
	 */
	public void setDateFormat(String dateFormat) {
		this.dateFormat = dateFormat;
	}
}
