/*
 * Copyright 2006 The Xanot team
 */
package org.xanot.structure;

/**
 * Propert Collection Setter rule. This rule define how to map sets of xml tag
 * with same name into object collection property.
 * 
 * @author Ferdinand Neman (newm4n _at_ gmail.com)
 */
public class PropertyCollectionSetRule extends XanotRule {
	private String xmlProperty = null;

	private String methodName = null;

	private String dateFormat = null;

	/**
	 * Creates a new PropertyCollectionSetRule object.
	 * 
	 * @param path
	 *            Path, should be equals to the parent element's tag name.
	 * @param xmlProperty
	 *            the xml tag name
	 * @param methodName
	 *            Method name tobe invoke for element addition.
	 */
	public PropertyCollectionSetRule(String path, String xmlProperty,
			String methodName) {
		super(path);
		setMethodName(methodName);
		setXmlProperty(xmlProperty);
	}

	/**
	 * Get the method name tobe invoke for element addition.
	 * 
	 * @return Returns the methodName.
	 */
	public String getMethodName() {
		return methodName;
	}

	/**
	 * Set the method name tobe invoke for element addition.
	 * 
	 * @param methodName
	 *            The methodName to set.
	 */
	public void setMethodName(String methodName) {
		this.methodName = methodName;
	}

	/**
	 * Get the xml element tag name
	 * 
	 * @return Returns the xmlProperty.
	 */
	public String getXmlProperty() {
		return xmlProperty;
	}

	/**
	 * Set the xml element tag name
	 * 
	 * @param xmlProperty
	 *            The xmlProperty to set.
	 */
	public void setXmlProperty(String xmlProperty) {
		this.xmlProperty = xmlProperty;
	}

	/**
	 * Get string representation of this object.
	 * 
	 * @return String representation.
	 */
	public String toString() {
		return "PropertyCollectionSetRule : [path:" + getPath()
				+ "],[xmlProperty:" + xmlProperty + "],[methodName:"
				+ methodName + "]";
	}

	/**
	 * Get the date format value.
	 * 
	 * @return Date format pattern
	 */
	public String getDateFormat() {
		return dateFormat;
	}

	/**
	 * Set the date format value
	 * 
	 * @param dateFormat
	 *            new date format pattern
	 */
	public void setDateFormat(String dateFormat) {
		this.dateFormat = dateFormat;
	}
}
