/*
 * Copyright 2006 The Xanot team
 */
package org.xanot.structure;

/**
 * Propert Setter rule. This rule define how to map sets of xml tag value into
 * object property.
 * 
 * @author Ferdinand Neman (newm4n _at_ gmail.com) <p/> TODO we might need add
 *         mandatory attributes as stated on the annotation.
 */
public class PropertySetRule extends XanotRule {
	private String propertyName = null;

	private String dateFormat = null;

	/**
	 * Creates a new PropertySetRule object.
	 * 
	 * @param path
	 *            Path, should be equals to the xml tag name.
	 * @param propertyName
	 *            Property name to set.
	 */
	public PropertySetRule(String path, String propertyName) {
		super(path);
		setPropertyName(propertyName);
	}

	/**
	 * Get the property name to set.
	 * 
	 * @return Returns the propertyName.
	 */
	public String getPropertyName() {
		return propertyName;
	}

	/**
	 * Set the property name to set.
	 * 
	 * @param propertyName
	 *            The propertyName to set.
	 */
	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
	}

	/**
	 * Get string representation of this object.
	 * 
	 * @return String representation.
	 */
	public String toString() {
		return "PropertySetRule : [path:" + getPath() + "],[property:"
				+ propertyName + "]";
	}

	/**
	 * Get the date format value.
	 * 
	 * @return Date format pattern
	 */
	public String getDateFormat() {
		return dateFormat;
	}

	/**
	 * Set the date format value
	 * 
	 * @param dateFormat
	 *            new date format pattern
	 */
	public void setDateFormat(String dateFormat) {
		this.dateFormat = dateFormat;
	}
}
